@@ -1,15 +1,14 @@
 """MD.ai Python client library."""
 
-try:
-    from importlib import metadata
-except ImportError:  # for Python<3.8
-    import importlib_metadata as metadata
+from importlib import metadata
+
 __version__ = metadata.version("mdai")
 
 from . import preprocess
 from .client import Client
 from .utils import common_utils
 from .utils import transforms
+from .utils import dicom_utils
 from .inference import delete_env, run_inference, infer
 
 try:
