@@ -2,6 +2,7 @@ import os
 import threading
 import re
 import math
+import json
 import uuid
 import zipfile
 import requests
@@ -9,6 +10,8 @@ import urllib3.exceptions
 from retrying import retry
 from tqdm import tqdm
 import arrow
+from pydicom.filereader import dcmread
+from .utils import dicom_utils
 from .preprocess import Project
 from . import __version__
 
@@ -40,6 +43,7 @@ class Client:
         self.access_token = access_token
         self.session = requests.Session()
         self._test_endpoint()
+        self.chat_completion = ChatCompletion(self.domain, self.session, self._create_headers())
 
     def project(
         self,
@@ -172,12 +176,15 @@ class Client:
         return None
 
     def load_model_annotations(self):
-        """Deprecated method: use `import_annotations` instead.
-        """
+        """Deprecated method: use `import_annotations` instead."""
         print("Deprecated method: use `import_annotations` instead.")
 
     def import_annotations(
-        self, annotations, project_id, dataset_id, chunk_size=ANNOTATIONS_IMPORT_DEFAULT_CHUNK_SIZE,
+        self,
+        annotations,
+        project_id,
+        dataset_id,
+        chunk_size=ANNOTATIONS_IMPORT_DEFAULT_CHUNK_SIZE,
     ):
         """Import annotations into project.
         For example, this method can be used to load machine learning model results into project as
@@ -236,6 +243,50 @@ class Client:
 
         return failed_annotations
 
+    def import_sr(self, sr_files, project_id="", dataset_id="", label_id=""):
+        """
+        Inputs:
+            `sr_files` - List of DICOM-SR files to load
+            `project_id` & `dataset_id` & `label_id` - Project information necessary to output SR to annotation note. All must be present if any are present. (optional)
+
+        Created by Dyllan Hofflich and MD.ai
+        """
+        if not project_id:
+            raise ValueError('Please add in the "project_id" argument')
+        if not dataset_id:
+            raise ValueError('Please add in the "dataset_id" argument')
+        if not label_id:
+            raise ValueError('Please add in the "label_id" argument')
+        annotations = []
+
+        for file_path in sr_files:
+            ds = dcmread(file_path)
+
+            # Get the referenced Dicom Files
+            referenced_dicoms = []
+            for study_seq in ds.CurrentRequestedProcedureEvidenceSequence:
+                referenced_study = {}
+                study_UID = study_seq.StudyInstanceUID
+                referenced_study["Study UID"] = study_UID
+                referenced_dicoms.append(referenced_study)
+
+            content_seq_list = list(ds.ContentSequence)
+
+            content = []
+            dicom_utils.iterate_content_seq(content, content_seq_list)
+
+            final_content = []
+            for annot in content:
+                annot = list(filter(None, annot))
+                final_content.append(" - ".join(annot))
+            for dicom_dict in referenced_dicoms:
+                study_uid = dicom_dict["Study UID"]
+                note = "\n".join(final_content)
+                annot_dict = {"labelId": label_id, "StudyInstanceUID": study_uid, "note": note}
+                annotations.append(annot_dict)
+
+        self.import_annotations(annotations, project_id, dataset_id)
+
     def _create_headers(self):
         headers = {}
         if self.access_token:
@@ -243,8 +294,7 @@ class Client:
         return headers
 
     def _test_endpoint(self):
-        """Checks endpoint for validity and authorization.
-        """
+        """Checks endpoint for validity and authorization."""
         test_endpoint = f"https://{self.domain}/api/test"
         r = self.session.get(test_endpoint, headers=self._create_headers())
         if r.status_code == 200:
@@ -259,8 +309,7 @@ class Client:
         stop_max_attempt_number=10,
     )
     def _gql(self, query, variables=None):
-        """Executes GraphQL query.
-        """
+        """Executes GraphQL query."""
         gql_endpoint = f"https://{self.domain}/api/graphql"
         headers = self._create_headers()
         headers["Accept"] = "application/json"
@@ -281,8 +330,7 @@ class Client:
 
 
 class ProjectDataManager:
-    """Manager for project data exports and downloads.
-    """
+    """Manager for project data exports and downloads."""
 
     def __init__(
         self,
@@ -392,8 +440,7 @@ class ProjectDataManager:
         stop_max_attempt_number=10,
     )
     def _check_data_export_job_progress(self):
-        """Poll for data export job progress.
-        """
+        """Poll for data export job progress."""
         endpoint = f"https://{self.domain}/api/data-export/{self.data_type}/progress"
         params = self._get_data_export_params()
         r = self.session.post(endpoint, json=params, headers=self.headers)
@@ -511,8 +558,7 @@ class ProjectDataManager:
             return os.path.join(self.path, dicom_metadata_fp)
 
     def _download_files(self, file_keys):
-        """Downloads exported files.
-        """
+        """Downloads exported files."""
         try:
             for file_key in file_keys:
                 print(f"Downloading file: {file_key}")
@@ -529,9 +575,12 @@ class ProjectDataManager:
 
                 # download file
                 # stream response so we can display progress bar
-                params = {"token": dl_token, "sessionId": dl_session_id}
                 url = f"https://{self.domain}/api/data-export/download/{key}"
-                r = requests.get(url, params=params, headers=self.headers, stream=True)
+                data = {"token": dl_token, "sessionId": dl_session_id}
+                r = requests.post(url, json=data, stream=True)
+                # fallback to GET if POST not available
+                if r.status_code == 405:
+                    r = requests.get(url, params=data, stream=True)
 
                 # total size in bytes
                 total_size = int(r.headers.get("content-length", 0))
@@ -565,8 +614,7 @@ class ProjectDataManager:
 
 
 class AnnotationsImportManager:
-    """Manager for importing annotations.
-    """
+    """Manager for importing annotations."""
 
     def __init__(
         self,
@@ -628,7 +676,6 @@ class AnnotationsImportManager:
             if r.status_code in (400, 401):
                 msg = "Provided IDs are invalid, or you do not have sufficient permissions."
                 print(msg)
-            self._on_job_error()
 
     def wait_until_ready(self):
         self._ready.wait()
@@ -640,19 +687,15 @@ class AnnotationsImportManager:
         stop_max_attempt_number=10,
     )
     def _check_job_progress(self):
-        """Poll for annotations import job progress.
-        """
+        """Poll for annotations import job progress."""
         endpoint = f"https://{self.domain}/api/data-import/annotations/progress"
         params = {"projectHashId": self.project_id, "jobId": self.job_id}
         r = self.session.post(endpoint, json=params, headers=self.headers)
-        if r.status_code != 200:
-            r.raise_for_status()
 
         try:
             body = r.json()
             status = body["status"]
         except (TypeError, KeyError):
-            self._on_job_error()
             return
 
         if status == "done":
@@ -701,14 +744,11 @@ class AnnotationsImportManager:
         endpoint = f"https://{self.domain}/api/data-import/annotations/done"
         params = {"projectHashId": self.project_id, "jobId": self.job_id}
         r = self.session.post(endpoint, json=params, headers=self.headers)
-        if r.status_code != 200:
-            r.raise_for_status()
 
         try:
             body = r.json()
             self.failed_annotations = body["failed"]
         except (TypeError, KeyError):
-            self._on_job_error()
             return
 
         num_failed = len(self.failed_annotations)
@@ -730,11 +770,76 @@ class AnnotationsImportManager:
         endpoint = f"https://{self.domain}/api/data-import/annotations/error"
         params = {"projectHashId": self.project_id, "jobId": self.job_id}
         r = self.session.post(endpoint, json=params, headers=self.headers)
-        if r.status_code != 200:
-            r.raise_for_status()
         print(
             f"Error importing annotations into project {self.project_id}"
             + f", dataset {self.dataset_id}."
         )
         # fire ready threading.Event
         self._ready.set()
+
+
+class ChatCompletion:
+    def __init__(self, domain="public.md.ai", session=None, headers=None):
+        self.domain = domain
+        if session and isinstance(session, requests.Session):
+            self.session = session
+        else:
+            self.session = requests.Session()
+        self.headers = headers
+        self.models = {"gpt-3.5-turbo": ['gpt-3.5-turbo-1106', 'gpt-3.5-turbo-0613', 'gpt-3.5-turbo-16k-0613', 'gpt-3.5-turbo-0301'],
+                       "gpt-4": ['gpt-4-1106-preview', 'gpt-4-vision-preview', 'gpt-4-0613', 'gpt-4-0314'],
+                       "llama2": ['togethercomputer/llama-2-70b-chat'],
+                       "mixtral": ['mistralai/Mixtral-8x7B-Instruct-v0.1']}
+
+    def list_models(self, model_name=None):
+        if model_name in self.models.keys():
+            print(f"{model_name} available model list: {self.models[model_name]}")
+        else:
+            for model in self.models.keys():
+                print(f"{model} available model list: {self.models[model]}")
+
+    def create(
+        self,
+        messages,
+        model="gpt-3.5-turbo",
+        functions=None,
+        function_call=None,
+        temperature=0,
+        top_p=None,
+        n=1,
+        stop=None,
+        max_tokens=None,
+        presence_penalty=None,
+        frequency_penalty=None,
+        logit_bias=None,
+    ):
+        """Creates a chat completion API call through MD.ai client."""
+        headers = self.headers
+        headers["Content-Type"] = "application/json"
+        data = {
+            "model": model,
+            "messages": messages,
+            "functions": functions,
+            "function_call": function_call,
+            "temperature": temperature,
+            "top_p": top_p,
+            "n": n,
+            "stop": stop,
+            "max_tokens": max_tokens,
+            "presence_penalty": presence_penalty,
+            "frequency_penalty": frequency_penalty,
+            "logit_bias": logit_bias,
+        }
+        try:
+            response = self.session.post(
+                f"https://{self.domain}/api/openai/chat/completions", json=data, headers=headers
+            )
+            response_json = json.loads(response.text)
+            if response_json.get("error"):
+                raise TypeError(f'{response_json.get("error")}')
+            response_content = response_json["response"]
+            return response_content
+        except Exception as error:
+            print(
+                f"{error}. Error Calling Chat Completion API. Please check all the parameters and try again"
+            )
